package burp.vaycore.onescan.common;

import burp.vaycore.common.config.ConfigManager;
import burp.vaycore.common.filter.FilterRule;
import burp.vaycore.common.log.Logger;
import burp.vaycore.common.utils.FileUtils;
import burp.vaycore.common.utils.GsonUtils;
import burp.vaycore.common.utils.PathUtils;
import burp.vaycore.common.utils.StringUtils;
import burp.vaycore.common.utils.Utils;
import burp.vaycore.onescan.manager.CollectManager;
import burp.vaycore.onescan.manager.FpManager;
import burp.vaycore.onescan.manager.WordlistManager;
import burp.vaycore.onescan.ui.widget.payloadlist.PayloadItem;
import burp.vaycore.onescan.ui.widget.payloadlist.PayloadRule;
import burp.vaycore.onescan.ui.widget.payloadlist.ProcessingItem;
import burp.vaycore.onescan.ui.widget.payloadlist.SimplePayloadList;
import com.google.gson.internal.LinkedTreeMap;
import java.io.File;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class Config {

   public static final String KEY_VERSION = "version";
   public static final String KEY_PAYLOAD_PROCESS_LIST = "payload-process-list";
   public static final String KEY_QPS_LIMIT = "qps-limit";
   public static final String KEY_REQUEST_DELAY = "request-delay";
   public static final String KEY_SCAN_LEVEL_DIRECT = "scan-level-direct";
   public static final String KEY_SCAN_LEVEL = "scan-level";
   public static final String KEY_RETRY_COUNT = "retry-count";
   public static final String KEY_MAX_DISPLAY_LENGTH = "max-display-length";
   public static final String KEY_COLLECT_PATH = "collect-path";
   public static final String KEY_EXCLUDE_SUFFIX = "exclude-suffix";
   public static final String KEY_INCLUDE_METHOD = "include-method";
   public static final String KEY_WORDLIST_PATH = "dict-path";
   public static final String KEY_DATABOARD_FILTER_RULES = "databoard-filter-rules";
   public static final String KEY_ENABLE_LISTEN_PROXY = "enable-listen-proxy";
   public static final String KEY_ENABLE_REMOVE_HEADER = "enable-remove-header";
   public static final String KEY_ENABLE_REPLACE_HEADER = "enable-replace-header";
   public static final String KEY_ENABLE_DIR_SCAN = "enable-dir-scan";
   public static final String KEY_ENABLE_PAYLOAD_PROCESSING = "payload-processing";
   public static final String DIRECT_LEFT = "left";
   public static final String DIRECT_RIGHT = "right";
   private static String sWorkDir;
   private static String sConfigPath;
   private static ConfigManager sConfigManager;


   public static void init(String wordDir) {
      sWorkDir = wordDir;
      sConfigPath = getWorkDir() + "config.json";
      sConfigManager = new ConfigManager(sConfigPath);
      initDefaultConfig("version", "2.0.0");
      initDefaultConfig("qps-limit", "1024");
      initDefaultConfig("request-delay", "0");
      initDefaultConfig("scan-level-direct", "left");
      initDefaultConfig("scan-level", "99");
      initDefaultConfig("retry-count", "3");
      initDefaultConfig("max-display-length", "0");
      initDefaultConfig("collect-path", getWorkDir() + "collect");
      initDefaultConfig("exclude-suffix", "3g2|3gp|7z|aac|abw|aif|aifc|aiff|arc|au|avi|azw|bin|bmp|bz|bz2|cmx|cod|csh|css|csv|doc|docx|eot|epub|gif|gz|ico|ics|ief|jar|jfif|jpe|jpeg|jpg|m3u|mid|midi|mjs|mp2|mp3|mpa|mpe|mpeg|mpg|mpkg|mpp|mpv2|odp|ods|odt|oga|ogv|ogx|otf|pbm|pdf|pgm|png|pnm|ppm|ppt|pptx|ra|ram|rar|ras|rgb|rmi|rtf|snd|svg|swf|tar|tif|tiff|ttf|vsd|wav|weba|webm|webp|woff|woff2|xbm|xls|xlsx|xpm|xul|xwd|zip|zip");
      initDefaultConfig("include-method", "GET|POST");
      initDefaultConfig("dict-path", getWorkDir() + "wordlist");
      initDefaultConfig("enable-listen-proxy", "false");
      initDefaultConfig("enable-remove-header", "false");
      initDefaultConfig("enable-replace-header", "true");
      initDefaultConfig("enable-dir-scan", "true");
      initDefaultConfig("payload-processing", "true");
      CollectManager.init(get("collect-path"));
      WordlistManager.init(get("dict-path"));
      initFpManager();
      onVersionUpgrade();
      onPreloadConfig();
   }

   private static void initFpManager() {
      String path = getWorkDir() + "fp_config.json";
      if(!FileUtils.isFile(path)) {
         InputStream is = Config.class.getClassLoader().getResourceAsStream("fp_config.json");
         String content = FileUtils.readStreamToString(is);
         FileUtils.writeFile(path, content);
      }

      FpManager.init(path);
   }

   private static void onVersionUpgrade() {
      String version = getVersion();
      if(!version.equals("2.0.0")) {
         putVersion("2.0.0");
         backupConfig(version);
         upgradeConfigKey();
         upgradeDomain();
         upgradeHeaders();
         upgradeRemoveHeaderList();
         upgradeWordlist();
         upgradePayloadProcessing(version);
      }

   }

   private static void backupConfig(String oldVersion) {
      if(oldVersion.startsWith("0.")) {
         String bakPath = sConfigPath + ".bak";
         String content = FileUtils.readFileToString(sConfigPath);
         FileUtils.writeFile(bakPath, content);
         Logger.info("Welcome to update version %s, the configuration file has been backed up to the %s", new Object[]{"2.0.0", bakPath});
      }
   }

   private static void upgradeConfigKey() {
      String enable;
      if(hasKey("enable-exclude-header")) {
         enable = get("enable-exclude-header");
         sConfigManager.put("enable-remove-header", enable);
         sConfigManager.remove("enable-exclude-header");
      }

      if(hasKey("white-host")) {
         enable = get("white-host");
         sConfigManager.put("host-allowlist", enable);
         sConfigManager.remove("white-host");
      }

      if(hasKey("black-host")) {
         enable = get("black-host");
         sConfigManager.put("host-blocklist", enable);
         sConfigManager.remove("black-host");
      }

      if(hasKey("exclude-headers")) {
         enable = get("exclude-headers");
         sConfigManager.put("remove-headers", enable);
         sConfigManager.remove("exclude-headers");
      }

      if(hasKey("hae-plugin-path")) {
         sConfigManager.remove("hae-plugin-path");
      }

   }

   private static void upgradeDomain() {
      String configJson = FileUtils.readFileToString(sConfigPath);
      if(configJson.contains("{{mdomain}}")) {
         configJson = configJson.replace("{{mdomain}}", "{{domain.name}}");
         boolean state = FileUtils.writeFile(sConfigPath, configJson);
         if(state) {
            Logger.info("Replace all {{mdomain}} to {{domain.name}} ok!");
            init(sWorkDir);
         }
      }

   }

   private static void upgradeHeaders() {
      List defaultHeaders = WordlistManager.getList("headers", "default");
      if(!defaultHeaders.isEmpty()) {
         String joinResult = StringUtils.join(defaultHeaders, ",");
         String headersMd5 = Utils.md5(joinResult.getBytes(StandardCharsets.UTF_8));
         String matchMd5 = "6eb466e03eda48b29275da941bfed84c";
         if(headersMd5.equals(matchMd5)) {
            InputStream is = Config.class.getClassLoader().getResourceAsStream("header.txt");
            ArrayList list = FileUtils.readStreamToList(is);
            WordlistManager.putList("headers", "default", list);
         }

      }
   }

   private static void upgradeRemoveHeaderList() {
      if(hasKey("remove-header-list")) {
         ArrayList list = getList("remove-header-list");
         WordlistManager.putList("remove-headers", list);
         sConfigManager.remove("remove-header-list");
      }

   }

   private static void upgradeWordlist() {
      ArrayList list;
      if(hasKey("payload-list")) {
         list = getList("payload-list");
         WordlistManager.putList("payload", list);
         sConfigManager.remove("payload-list");
      }

      if(hasKey("header-list")) {
         list = getList("header-list");
         WordlistManager.putList("headers", list);
         sConfigManager.remove("header-list");
      }

      if(hasKey("user-agent-list")) {
         list = getList("user-agent-list");
         WordlistManager.putList("user-agent", list);
         sConfigManager.remove("user-agent-list");
      }

      if(hasKey("whitelist")) {
         list = getList("whitelist");
         WordlistManager.putList("host-allowlist", list);
         sConfigManager.remove("whitelist");
      }

      if(hasKey("blacklist")) {
         list = getList("blacklist");
         WordlistManager.putList("host-blocklist", list);
         sConfigManager.remove("blacklist");
      }

      if(hasKey("exclude-header")) {
         list = getList("exclude-header");
         WordlistManager.putList("remove-headers", list);
         sConfigManager.remove("exclude-header");
      }

      if(hasKey("web-name-collect-path")) {
         sConfigManager.remove("web-name-collect-path");
      }

      if(hasKey("json-field-collect-path")) {
         sConfigManager.remove("json-field-collect-path");
      }

   }

   private static void upgradePayloadProcessing(String oldVersion) {
      String version = oldVersion.replace(".", "");
      int versionInt = StringUtils.parseInt(version);
      if(versionInt != 0 && versionInt <= 137) {
         if(hasKey("merge-payload-processing")) {
            boolean items = getBoolean("merge-payload-processing");
            put("payload-processing", (String)String.valueOf(items));
            sConfigManager.remove("merge-payload-processing");
         }

         if(hasKey("payload-process-list")) {
            ArrayList items1 = (ArrayList)sConfigManager.get("payload-process-list");
            ArrayList newItems = new ArrayList();
            if(items1 != null && !items1.isEmpty()) {
               ArrayList result = mapItemsConvert(items1);
               ProcessingItem item = new ProcessingItem();
               item.setEnabled(true);
               item.setItems(result);
               item.setName("Low version rules");
               newItems.add(item);
            }

            sConfigManager.put("payload-process-list", newItems);
         }

      }
   }

   private static void onPreloadConfig() {
      preparePayloadProcessList();
      prepareDataboardFilterRules();
   }

   private static void initDefaultConfig(String key, String defValue) {
      if(!hasKey(key)) {
         sConfigManager.put(key, defValue);
      }

   }

   private static void checkInit() {
      if(sConfigManager == null) {
         throw new IllegalStateException("Config class is not initialize");
      }
   }

   public static String getWorkDir() {
      return StringUtils.isNotEmpty(sWorkDir) && FileUtils.isDir(sWorkDir)?sWorkDir:PathUtils.getUserHome() + ".config" + File.separator + "OneScan" + File.separator;
   }

   public static String getVersion() {
      return get("version");
   }

   public static void putVersion(String version) {
      put("version", (String)version);
   }

   public static String getFilePath(String key) {
      return getFilePath(key, false);
   }

   public static String getFilePath(String key, boolean isDir) {
      checkInit();
      String path = (String)sConfigManager.get(key);
      if(StringUtils.isEmpty(path)) {
         return path;
      } else {
         File dir = isDir?new File(path):(new File(path)).getParentFile();
         if(!dir.exists()) {
            boolean mkdirs = dir.mkdirs();
            Logger.debug("Config item path mkdirs: %s", new Object[]{Boolean.valueOf(mkdirs)});
         }

         return path;
      }
   }

   public static void put(String key, String text) {
      checkInit();
      sConfigManager.put(key, text);
   }

   public static void put(String key, Object obj) {
      checkInit();
      sConfigManager.put(key, obj);
   }

   public static String get(String key) {
      checkInit();
      return (String)sConfigManager.get(key);
   }

   public static boolean getBoolean(String key) {
      checkInit();
      String value = (String)sConfigManager.get(key);
      return "true".equals(value);
   }

   public static ArrayList getList(String key) {
      checkInit();
      return sConfigManager.getList(key);
   }

   public static boolean hasKey(String key) {
      checkInit();
      return sConfigManager.hasKey(key);
   }

   public static void removeKey(String key) {
      checkInit();
      sConfigManager.remove(key);
   }

   private static void preparePayloadProcessList() {
      try {
         ArrayList items = (ArrayList)sConfigManager.get("payload-process-list");
         ArrayList result = new ArrayList();
         if(items != null && !items.isEmpty()) {
            Iterator var2 = items.iterator();

            while(var2.hasNext()) {
               LinkedTreeMap mapItem = (LinkedTreeMap)var2.next();
               ProcessingItem item = new ProcessingItem();
               item.setEnabled(((Boolean)mapItem.get("enabled")).booleanValue());
               String mergeValue = String.valueOf(mapItem.get("merge"));
               item.setMerge(Boolean.parseBoolean(mergeValue));
               String name = String.valueOf(mapItem.get("name"));
               item.setName(name);
               ArrayList payloadMapItems = (ArrayList)mapItem.get("items");
               ArrayList payloadItems = mapItemsConvert(payloadMapItems);
               item.setItems(payloadItems);
               result.add(item);
            }
         }

         put("payload-process-list", (Object)result);
      } catch (Exception var9) {
         ;
      }

   }

   private static ArrayList mapItemsConvert(ArrayList items) {
      ArrayList result = new ArrayList();
      if(items != null && !items.isEmpty()) {
         Iterator var2 = items.iterator();

         while(var2.hasNext()) {
            LinkedTreeMap mapItem = (LinkedTreeMap)var2.next();
            PayloadItem item = new PayloadItem();
            Double scope = (Double)mapItem.get("scope");
            item.setScope(scope.intValue());
            PayloadRule rule = SimplePayloadList.getPayloadRuleByType((String)mapItem.get("ruleType"));
            if(rule != null) {
               LinkedTreeMap ruleMap = (LinkedTreeMap)mapItem.get("rule");
               ArrayList ruleParamValues = (ArrayList)ruleMap.get("paramValues");

               for(int j = 0; j < rule.paramCount(); ++j) {
                  String paramValue = (String)ruleParamValues.get(j);
                  rule.setParamValue(j, paramValue);
               }

               item.setRule(rule);
               result.add(item);
            }
         }

         return result;
      } else {
         return result;
      }
   }

   private static void prepareDataboardFilterRules() {
      Object obj = sConfigManager.get("databoard-filter-rules");
      if(obj == null) {
         obj = new ArrayList();
      }

      String json = GsonUtils.toJson(obj);
      ArrayList rules = GsonUtils.toList(json, FilterRule.class);
      put("databoard-filter-rules", (Object)rules);
   }

   public static ArrayList getPayloadProcessList() {
      checkInit();
      return (ArrayList)sConfigManager.get("payload-process-list");
   }

   public static ArrayList getDataboardFilterRules() {
      checkInit();
      return (ArrayList)sConfigManager.get("databoard-filter-rules");
   }
}
